<?php

namespace Battles;

use Battles\Database\DBPDO;
use Exceptions\GameException;

class ShopItem extends Item
{
    private const NO_ITEMS_IN_STOCK = "Товара нет в наличии!";
    private const NO_MONEY = "У вас нет денег!";
    private const BUTTON = [
        'setmarket' => 'Сдать в магазин',
        'unsetmarket' => 'Снять с продажи',
        'buymarket' => 'Совершить обмен',
        'sellshop' => 'Продать',
        'buyshop' => 'Купить',
    ];
    private const BUY_QUERY = <<<SQL
insert into inventory (
    owner_id, name, item_type, durability,
    need_strength, need_dexterity, need_intuition, need_endurance, need_intelligence, need_wisdom,
    add_strength, add_dexterity, add_intuition, add_endurance, add_intelligence, add_wisdom,
    add_accuracy, add_evasion, add_criticals, add_min_physical_damage, add_max_physical_damage,
    image, weight, price) 
select 
    ?, name, item_type, durability,
    need_strength, need_dexterity, need_intuition, need_endurance, need_intelligence, need_wisdom,
    add_strength, add_dexterity, add_intuition, add_endurance, add_intelligence, add_wisdom,
    add_accuracy, add_evasion, add_criticals, add_min_physical_damage, add_max_physical_damage,
    image, weight, greatest(
        (
            (add_strength + add_dexterity + add_intuition + add_endurance + add_intelligence + add_wisdom) * 
            (5 + floor((add_strength + add_dexterity + add_intuition + add_endurance + add_intelligence + add_wisdom) / 10)) 
        ) + 
        ( 
            (add_accuracy + add_criticals + add_evasion) * 
            (2 + floor((add_accuracy + add_criticals + add_evasion) / 50)) 
        ) + 
        ( 
            (add_min_physical_damage + add_max_physical_damage) * 
            (1 + floor((add_min_physical_damage + add_max_physical_damage) / 100))
        )
    ,1) 
from items where id = ?
SQL;

    // Тип операции в магазине. Для отображения разных блоков в разных случаях.
    private $optype;
    private ?int $shop_item_quantity;
    private ?int $price;
    public static string $status = '';

    public function __construct($row, $operationType = null)
    {
        parent::__construct($row);
        if ($operationType) {
            $this->optype = $operationType;
        }
        $this->price = $row->price ?? null;
        $this->shop_item_quantity = $row->shop_item_quantity ?? null;
        $this->item_id = $row->item_id ?? $row->id;
    }

    public function printInfo(): string
    {
        //$this->printAllInfo();
        $str = $this->getAllInfo();
        if ($this->optype === 'buyshop' && $this->shop_item_quantity > 0 && $this->shop_item_quantity < 20) {
            $str .= "<div style='margin-top: 9px; font-style: italic;'>На складе осталось $this->shop_item_quantity единиц товара!</div>";
        }
        if ($this->optype === 'sellshop') {
            if ($this->getSellPriceMean() < 50) {
                $goods = 'этот хлам';
            } elseif ($this->getSellPriceMean() < 100) {
                $goods = 'этот посредственный товар';
            } elseif ($this->getSellPriceMean() < 500) {
                $goods = 'этот неплохой предмет';
            } elseif ($this->getSellPriceMean() < 1000) {
                $goods = 'эту отличную штуку';
            } else {
                $goods = 'это превосходное изделие';
            }
            $str .= "<div style='margin-top: 9px; font-style: italic;'>В среднем за $goods можно выручить <span class='success'>{$this->getSellPriceMean()}</span> кр.</div>";
        }
        return $str;
    }

    public function printImage(): string
    {
        if (!$this->image) {
            $this->image = 'noitem.png';
        }
        return "<img src='/i/sh/$this->image' class='item-wrap-normal' alt=''>";
    }

    //todo наличка после покупки отображается с задержкой.
    public static function buyItem($id, User $buyer)
    {
        $db = new DBPDO();
        $check = $db->ofetch("select * from trade_offers where shop_item_id = ?", $id);
        if (empty($check->shop_item_quantity) || empty($check->shop_item_id)) {
            self::$status = self::NO_ITEMS_IN_STOCK;
        }

        // TODO БАРТЕР!
        if (isset($check->barter_item_list_json)) {
            self::$status = "Работаем по бартеру!";
        }

        $db->execute(self::BUY_QUERY, [$buyer->getId(), $check->shop_item_id]);
        $item = $db->ofetch("select item_id, name, price from inventory where item_id = ?", $db->lastInsertId());
        if (empty($item->item_id) || empty($item->name)) {
            self::$status = 'Запрос в базу не прошёл.';
        } else {
            $user = new User($_SESSION['uid']);
            // Если не хватает налички, снимаем с банка с комиссией.
            if ($user->getMoney() < $item->price) {
                try {
                    $bank = new Bank($buyer->getId());
                    $bank->withdrawMoney($item->price);
                } catch (GameException $e) {
                    self::$status = 'Банковская ошибка! ' . self::NO_MONEY;
                }
            } else {
                $user->setMoney($user->getMoney() - $item->price);
                $user->saveMoney();
            }
        }

        if ($check->shop_item_quantity != -1) {
            $db->execute("update trade_offers set shop_item_quantity = shop_item_quantity -1 where shop_item_id = ?", $check->shop_item_id);
        }

        $deloText = $buyer->getLogin() . " купил товар «" . $item->name . "» id:(" . $item->item_id . ") в магазине за " . $item->price . ".";
        GameLogs::addUserLog($buyer->getId(), $deloText);
        self::$status = "Предмет " . $item->name . " куплен за " . $item->price . ".";
    }

    public static function sellItem($id, User $seller, $bankTrade = 0)
    {
        $db = new DBPDO();
        $item = $db->ofetch('select * from inventory where item_id = ?', $id);
        $sellingItemName = $item->name;
        // Продажа за цену от нуля до половины стоимости.
        $sellingPrice = $item->price > 1 ? mt_rand(0, $item->price / 2) : mt_rand(0, 1);
        $db->execute('delete from inventory where item_id = ?', $id);
        if ($bankTrade) {
            $bank = new Bank($seller->getId());
            $bank->setMoney($bank->getMoney() + $sellingPrice);
            Bank::setBankMoney($bank->getMoney(), $seller->getId(), 'sellShop');
        } else {
            $db->execute('update users set money = money - ? where id = ?', [$sellingPrice, $_SESSION['uid']]);
        }
        $deloText = "{$seller->getLogin()} продал товар «{$sellingItemName}» id:($id) в магазине за $sellingPrice кр.";
        GameLogs::addUserLog($seller->getId(), $deloText);
        if ($sellingPrice == 0) {
            self::$status = "После длительных и изнурительных торгов вы плюнули на всё и просто подарили ваш «{$sellingItemName}» торговцу.";
        } else {
            self::$status = "Вы продали «{$sellingItemName}» за $sellingPrice кр.";
        }
    }

    /** Подчсчёт средней суммы продажи.
     * @return int
     */
    private function getSellPriceMean(): ?int
    {
        if ($this->price > 1) {
            $arr = range(0, $this->price / 2);
            return array_sum($arr) / count($arr);
        } else {
            return $this->price == 1 ? 1 : null;
        }
    }

    /**
     * Для кнопок управления под картинкой предмета в зависимости от ситуации.
     */
    public function printControls(): string
    {
        if (!in_array($this->optype, ['setmarket', 'unsetmarket', 'buymarket', 'sellshop', 'buyshop',])) {
            return '';
        }
        $str = $this->optype == 'setmarket' ? '<input placeholder=" ' . $this->price . ' " name="cost">' : '';
        $button_name = self::BUTTON[$this->optype];
        return <<<FORM
<form method="post">$str
<input type="hidden" name="itemId" value="$this->item_id">
<br><input type="submit" name="$this->optype" value="$button_name">
</form>
FORM;
    }

    /**
     * @return int
     */
    public function getItemType(): int
    {
        return $this->item_type;
    }

    /** Выдача магазинных предметов по запросу.
     * Ввелась чтобы перебить takeshopitem() в functions с идентичным функционалом.
     * @param int $item_id ИД предмета.
     * @param int $to ИД пперсонажа-получателя.
     */
    public static function giveNewItem(int $item_id, int $to): array
    {
        $check = DBPDO::$db->ofetch('select 1 from items where id = ?', $item_id);
        if (!$check) {
            return [];
        }
        DBPDO::$db->execute(self::BUY_QUERY, [$to, $item_id]);
        $return = DBPDO::$db->ofetch('select image, name from inventory where item_id = ?', DBPDO::$db->lastInsertId());
        return [
            'img' => $return->image,
            'name' => $return->name,
            'id' => $item_id,
        ];
    }

}